// src/managers/datetimeManager.ts

import { Server } from 'socket.io'
import { appLogger } from '../utilities/logger'
import { getRootPath } from '../utilities/storage'
import { readJsonValue, setJsonValue } from '../utilities/json'

class DatetimeManager {
  private static readonly GAME_SPEED = 8 // 24 game hours / 3 real hours
  private static readonly UPDATE_INTERVAL = 1000 // 1 second

  private io: Server | null = null
  private intervalId: NodeJS.Timeout | null = null
  private currentDateTime: Date = new Date()

  public async boot(io: Server): Promise<void> {
    this.io = io
    await this.loadDateTime()
    this.startDateTimeLoop()
    appLogger.info('Datetime manager loaded')
  }

  public stop(): void {
    if (this.intervalId) {
      clearInterval(this.intervalId)
      this.intervalId = null
    }
  }

  private async loadDateTime(): Promise<void> {
    try {
      const datetimeString = await readJsonValue<string>(this.getWorldFilePath(), 'datetime')
      this.currentDateTime = new Date(datetimeString)
    } catch (error) {
      appLogger.error(`Failed to load datetime: ${error instanceof Error ? error.message : String(error)}`)
      this.currentDateTime = new Date() // Use current date as fallback
    }
  }

  private startDateTimeLoop(): void {
    this.intervalId = setInterval(() => {
      this.advanceGameTime()
      this.emitDateTime()
      this.saveDateTime()
    }, DatetimeManager.UPDATE_INTERVAL)
  }

  private advanceGameTime(): void {
    const advanceMilliseconds = DatetimeManager.GAME_SPEED * DatetimeManager.UPDATE_INTERVAL
    this.currentDateTime = new Date(this.currentDateTime.getTime() + advanceMilliseconds)
  }

  private emitDateTime(): void {
    this.io?.emit('datetime', this.formatDateTime(this.currentDateTime))
  }

  private formatDateTime(date: Date): string {
    return date.toISOString().slice(0, 19).replace('T', ' ')
  }

  private async saveDateTime(): Promise<void> {
    try {
      await setJsonValue(this.getWorldFilePath(), 'datetime', this.formatDateTime(this.currentDateTime))
    } catch (error) {
      appLogger.error(`Failed to save datetime: ${error instanceof Error ? error.message : String(error)}`)
    }
  }

  private getWorldFilePath(): string {
    return getRootPath('data', 'world.json')
  }
}

export default new DatetimeManager()